@@ -1,4 +1,5 @@
 #Copyright (C) 2008 Codethink Ltd
+#Copyright (C) 2010 Novell, Inc.
 
 #This library is free software; you can redistribute it and/or
 #modify it under the terms of the GNU Lesser General Public
@@ -12,11 +13,10 @@
 #along with this program; if not, write to the Free Software
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-from interfaces import *
-from enum import Enum
-from accessible import Accessible
+from gi.repository import Atspi
+from enum import *
+import utils
 
-from dbus.types import UInt32, Int32
 
 __all__ = [
            "Text",
@@ -73,7 +73,7 @@ TEXT_CLIP_NONE = TEXT_CLIP_TYPE(0)
 
 #------------------------------------------------------------------------------
 
-class Text(Accessible):
+class Text:
         """
         The text interface should be implemented by objects which place
         textual information onscreen as character strings or glyphs.
@@ -100,6 +100,9 @@ class Text(Accessible):
         and the set returned at a particular character offset via Text::getAttributeRun.
         """
 
+        def __init__(self, obj):
+                self.obj = obj
+
         def addSelection(self, index):
                 """
                 The result of calling addSelection on objects which already have
@@ -112,8 +115,7 @@ class Text(Accessible):
                 other reasons (for instance if the user does not have permission
                 to copy the text into the relevant selection buffer).
                 """
-                func = self.get_dbus_method("AddSelection", dbus_interface=ATSPI_TEXT)
-                return func(index)
+                return Atspi.Text.add_selection(self.obj, index)
 
         def getAttributeRun(self, offset):
                 """
@@ -164,8 +166,7 @@ class Text(Accessible):
                 @return the AttributeSet defined at offset, optionally including
                 the 'default' attributes.
                 """
-                func = self.get_dbus_method("GetAttributeRun", dbus_interface=ATSPI_TEXT)
-                [attrs, startOffset, endOffset] = func(offset, includeDefaults)
+                [attrs, startOffset, endOffset] = Atspi.Text.get_attribute_run(self.obj, offset)
                 dict = [key + ':' + value for key, value in attrs.values()]
                 return [dict, startOffset, endOffset]
 
@@ -191,8 +192,7 @@ class Text(Accessible):
                 @return the value of attribute (name-value pair) corresponding
                 to "name", if defined.
                 """
-                func = self.get_dbus_method("GetAttributeValue", dbus_interface=ATSPI_TEXT)
-                return func(offset, attributeName)
+                return Atspi.Text.get_attribute_value(self.obj, offset, attributeName)
 
         def getAttributes(self, offset):
                 """
@@ -200,14 +200,12 @@ class Text(Accessible):
                 @return the attributes at offset, as a semicolon-delimited set
                 of colon-delimited name-value pairs.
                 """
-                func = self.get_dbus_method("GetAttributes", dbus_interface=ATSPI_TEXT)
-                [attrs, startOffset, endOffset] = func(Int32(offset))
+                [attrs, startOffset, endOffset] = Atspi.Text.get_attributes(self.obj, offset)
                 arr = [key + ':' + value for key, value in attrs.items()]
 		str = ';'.join (arr)
                 return [str, startOffset, endOffset]
 
         def getBoundedRanges(self, x, y, width, height, coordType, xClipType, yClipType):
-                #TODO Return a list of range structures
                 """
                 Return the text content within a bounding box, as a list of Range
                 structures. Depending on the TEXT_CLIP_TYPE parameters, glyphs
@@ -235,8 +233,7 @@ class Text(Accessible):
                 determines whether text which intersects the bounding box in
                 the y direction is included.
                 """
-                func = self.get_dbus_method("GetBoundedRanges", dbus_interface=ATSPI_TEXT)
-                return func(x, y, width, height, UInt32(coordType), xClipType, yClipType)
+                return Atspi.Text.get_bounded_ranges(self.obj, x, y, width, height, coordType, xClipType, yClipType)
 
         def getCharacterAtOffset(self, offset):
                 """
@@ -246,10 +243,9 @@ class Text(Accessible):
                 UCS-4 representation of the character at the specified text offset,
                 or 0 if offset is out of range.
                 """
-                func = self.get_dbus_method("GetCharacterAtOffset", dbus_interface=ATSPI_TEXT)
-                return func(offset)
+                return Atspi.Text.get_character_offset(self.obj, offset)
 
-        def getCharacterExtents(self, offset, x, y, width, height, coordType):
+        def getCharacterExtents(self, offset, coordType):
                 """
                 Obtain a the bounding box, as x, y, width, and height, of the
                 character or glyph at a particular character offset in this object's
@@ -260,18 +256,6 @@ class Text(Accessible):
                 and therefore may apply to more than one character offset. 
                 @param : offset
                 the character offset of the character or glyph being queried.
-                @param : x
-                the minimum horizontal coordinate of the bounding box of the
-                glyph representing the character at offset. 
-                @param : y
-                the minimum vertical coordinate of the bounding box of the glyph
-                representing the character at offset. 
-                @param : width
-                the horizontal extent of the bounding box of the glyph representing
-                the character at offset. 
-                @param : height
-                the vertical extent of the bounding box of the glyph representing
-                the character at offset. 
                 @param : coordType
                 If 0, the results will be reported in screen coordinates, i.e.
                 in pixels relative to the upper-left corner of the screen, with
@@ -280,8 +264,8 @@ class Text(Accessible):
                 window, with the x axis pointing right and the y axis pointing
                 down.
                 """
-                func = self.get_dbus_method("GetCharacterExtents", dbus_interface=ATSPI_TEXT)
-                return func(offset, x, y, width, height, UInt32(coordType))
+                ret = Atspi.Text.get_character_extents(self.obj, offset, coordType)
+                return utils.rectToList(ret)
 
         def getDefaultAttributeSet(self):
                 """
@@ -294,8 +278,8 @@ class Text(Accessible):
                 whereas an object whose text weight is inspecified may report
                 the default or implied text weight in the default AttributeSet.
                 """
-                func = self.get_dbus_method("GetDefaultAttributeSet", dbus_interface=ATSPI_TEXT)
-                return [key + ':' + value for key, value in func().values()]
+                ret = Atspi.Text.get_default_attribute_set(self.obj)
+                return [key + ':' + value for key, value in ret.values()]
 
         def getDefaultAttributes(self):
                 """
@@ -303,8 +287,8 @@ class Text(Accessible):
                 @return the attributes which apply to the entire text content,
                 but which were not explicitly specified by the content creator.
                 """
-                func = self.get_dbus_method("GetDefaultAttributes", dbus_interface=ATSPI_TEXT)
-                return ';'.join([key + ':' + value for key, value in func().iteritems()])
+                ret = Atspi.Text.get_default_attributes(self.obj)
+                return ';'.join([key + ':' + value for key, value in ret.iteritems()])
 
         def getNSelections(self):
                 """
@@ -318,8 +302,7 @@ class Text(Accessible):
                 @return the number of contiguous selections in the current Text
                 object.
                 """
-                func = self.get_dbus_method("GetNSelections", dbus_interface=ATSPI_TEXT)
-                return func()
+                return Atspi.Text.get_n_selections(self.obj)
 
         def getOffsetAtPoint(self, x, y, coordType):
                 """
@@ -336,8 +319,7 @@ class Text(Accessible):
                 of the glyph whose onscreen bounds contain the point x,y, or
                 -1 if the point is outside the bounds of any glyph.
                 """
-                func = self.get_dbus_method("GetOffsetAtPoint", dbus_interface=ATSPI_TEXT)
-                return func(x, y, UInt32(coordType))
+                return Atspi.Text.get_offset_at_point(self.obj, x, y, UInt32(coordType))
 
         def getRangeExtents(self, startOffset, endOffset, coordType):
                 """
@@ -367,8 +349,8 @@ class Text(Accessible):
                 corner of the screen; if 1, the coordinates are reported relative
                 to the corner of the containing toplevel window.
                 """
-                func = self.get_dbus_method("GetRangeExtents", dbus_interface=ATSPI_TEXT)
-                return func(startOffset, endOffset, UInt32(coordType))
+                ret = Atspi.Text.get_range_extents(self.obj, startOffset, endOffset, coordType)
+                return utils.rectToList(ret)
 
         def getSelection(self, selectionNum):
                 """
@@ -384,8 +366,8 @@ class Text(Accessible):
                 back-filled with the offset of the character immediately following
                 the resulting substring, if one exists. 
                 """
-                func = self.get_dbus_method("GetSelection", dbus_interface=ATSPI_TEXT)
-                return func(selectionNum)
+                ret = Atspi.Text.get_selection(self.obj, selectionNum)
+                return rangeToList(ret)
 
         def getText(self, startOffset, endOffset):
                 """
@@ -402,10 +384,9 @@ class Text(Accessible):
                 startOffset (inclusive) up to but not including the character
                 at endOffset.
                 """
-                func = self.get_dbus_method("GetText", dbus_interface=ATSPI_TEXT)
                 if not endOffset:
                         endOffset = -1
-                return func(Int32(startOffset), Int32(endOffset))
+                return Atspi.Text.get_text(self.obj, startOffset, endOffset)
 
         def getTextAfterOffset(self, offset, type):
                 """
@@ -432,8 +413,8 @@ class Text(Accessible):
                 @return a string which is a substring of the text content of
                 the object, delimited by the specified boundary condition.
                 """
-                func = self.get_dbus_method("GetTextAfterOffset", dbus_interface=ATSPI_TEXT)
-                return func(offset, type)
+                ret = Atspi.Text.get_text_after_offset(self.obj, offset, type)
+                return textRangeToList(ret)
 
         def getTextAtOffset(self, offset, type):
                 """
@@ -459,8 +440,8 @@ class Text(Accessible):
                 @return a string which is a substring of the text content of
                 the object, delimited by the specified boundary condition.
                 """
-                func = self.get_dbus_method("GetTextAtOffset", dbus_interface=ATSPI_TEXT)
-                return func(offset, type)
+                ret = Atspi.Text.get_text_at_offset(self.obj, offset, type)
+                return textRangeToList(ret)
 
         def getTextBeforeOffset(self, offset, type):
                 """
@@ -487,7 +468,8 @@ class Text(Accessible):
                 the object, delimited by the specified boundary condition.
                 """
                 func = self.get_dbus_method("GetTextBeforeOffset", dbus_interface=ATSPI_TEXT)
-                return func(offset, type)
+                ret = Atspi.Text.get_text_before_offset(self.obj, offset, type)
+                return textRangeToList(ret)
 
         def removeSelection(self, selectionNum):
                 """
@@ -499,8 +481,7 @@ class Text(Accessible):
                 @return True if the selection was successfully removed, False
                 otherwise.
                 """
-                func = self.get_dbus_method("RemoveSelection", dbus_interface=ATSPI_TEXT)
-                return func(selectionNum)
+                return Atspi.Text.remove_selection(self.obj, index)
 
         def setCaretOffset(self, offset):
                 """
@@ -513,8 +494,7 @@ class Text(Accessible):
                 @return TRUE if the request was carried out, or FALSE if the
                 caret could not be moved to the requested position.
                 """
-                func = self.get_dbus_method("SetCaretOffset", dbus_interface=ATSPI_TEXT)
-                return func(offset)
+                return Atspi.Text.set_caret_offset(self.obj, offset)
 
         def setSelection(self, selectionNum, startOffset, endOffset):
                 """
@@ -532,11 +512,10 @@ class Text(Accessible):
                 @return True if the selection corresponding to selectionNum is
                 successfully modified, False otherwise.
                 """
-                func = self.get_dbus_method("setSelection", dbus_interface=ATSPI_TEXT)
-                return func(selectionNum, startOffset, endOffset)
+                return Atspi.Text.set_selection(self.obj, selectionNum, startOffset, endOffset)
 
         def get_caretOffset(self):
-                return Int32(self._pgetter(ATSPI_TEXT, "CaretOffset"))
+                return Atspi.Text.get_caret_offset(self.obj)
         _caretOffsetDoc = \
                 """
                 The current offset of the text caret in the Text object. This
@@ -549,7 +528,7 @@ class Text(Accessible):
         caretOffset = property(fget=get_caretOffset, doc=_caretOffsetDoc)
 
         def get_characterCount(self):
-                return Int32(self._pgetter(ATSPI_TEXT, "CharacterCount"))
+                return Atspi.Text.get_character_count(self.obj)
         _characterCountDoc = \
                 """
                 The total current number of characters in the Text object, including
@@ -557,31 +536,8 @@ class Text(Accessible):
                 """
         characterCount = property(fget=get_characterCount, doc=_characterCountDoc)
 
-        class Range(list):
-                def __new__(cls, startOffset, endOffset, content, data):
-                        list.__new__(cls, (startOffset, endOffset, content, data))
-                def __init__(self, startOffset, endOffset, content, data):
-                        list.__init__(self, (startOffset, endOffset, content, data))
+def rangeToList(r):
+        return (r.start_offset, r.end_offset)
 
-                def _get_startOffset(self):
-                        return self[0]
-                def _set_startOffset(self, val):
-                        self[0] = val
-                startOffset = property(fget=_get_startOffset, fset=_set_startOffset)
-                def _get_endOffset(self):
-                        return self[1]
-                def _set_endOffset(self, val):
-                        self[1] = val
-                endOffset = property(fget=_get_endOffset, fset=_set_endOffset)
-                def _get_content(self):
-                        return self[2]
-                def _set_content(self, val):
-                        self[2] = val
-                content = property(fget=_get_content, fset=_set_content)
-                def _get_data(self):
-                        return self[3]
-                def _set_data(self, val):
-                        self[3] = val
-                data = property(fget=_get_data, fset=_set_data)
-
-#END----------------------------------------------------------------------------
+def textRangeToList(r):
+        return (r.content, r.start_offset, r.end_offset)
